@@ -44,8 +44,9 @@
 #include <gudev/gudev.h>
 #endif
 
+#include "mpris-controller.h"
 #include "gnome-settings-plugin.h"
-#include "gnome-settings-session.h"
+#include "gnome-settings-bus.h"
 #include "gnome-settings-profile.h"
 #include "gsd-marshal.h"
 #include "gsd-media-keys-manager.h"
@@ -68,10 +69,6 @@
 #define GNOME_KEYRING_DBUS_PATH "/org/gnome/keyring/daemon"
 #define GNOME_KEYRING_DBUS_INTERFACE "org.gnome.keyring.Daemon"
 
-#define GS_DBUS_NAME                            "org.gnome.ScreenSaver"
-#define GS_DBUS_PATH                            "/org/gnome/ScreenSaver"
-#define GS_DBUS_INTERFACE                       "org.gnome.ScreenSaver"
-
 #define SHELL_DBUS_NAME "org.gnome.Shell"
 #define SHELL_DBUS_PATH "/org/gnome/Shell"
 
@@ -79,6 +76,9 @@
 
 #define SHELL_GRABBER_RETRY_INTERVAL 1
 
+/* Screencasts last for 30 seconds */
+#define SCREENCAST_LENGTH 30
+
 static const gchar introspection_xml[] =
 "<node name='/org/gnome/SettingsDaemon/MediaKeys'>"
 "  <interface name='org.gnome.SettingsDaemon.MediaKeys'>"
@@ -171,6 +171,14 @@ struct GsdMediaKeysManagerPrivate
         GCancellable    *shell_cancellable;
         GCancellable    *grab_cancellable;
 
+        /* ScreenSaver stuff */
+        GsdScreenSaver  *screen_saver_proxy;
+
+        /* Screencast stuff */
+        GDBusProxy      *screencast_proxy;
+        guint            screencast_timeout_id;
+        GCancellable    *screencast_cancellable;
+
         /* systemd stuff */
         GDBusProxy      *logind_proxy;
         gint             inhibit_keys_fd;
@@ -184,6 +192,8 @@ struct GsdMediaKeysManagerPrivate
         GCancellable    *cancellable;
 
         guint            start_idle_id;
+
+        MprisController *mpris_controller;
 };
 
 static void     gsd_media_keys_manager_class_init  (GsdMediaKeysManagerClass *klass);
@@ -842,7 +852,7 @@ gnome_session_shutdown (GsdMediaKeysManager *manager)
 	GVariant *variant;
         GDBusProxy *proxy;
 
-        proxy = gnome_settings_session_get_session_proxy ();
+        proxy = G_DBUS_PROXY (gnome_settings_bus_get_session_proxy ());
 	variant = g_dbus_proxy_call_sync (proxy,
 					  "Shutdown",
 					  NULL,
@@ -1006,23 +1016,35 @@ do_touchpad_action (GsdMediaKeysManager *manager)
         g_object_unref (settings);
 }
 
+static void
+on_screen_locked (GsdScreenSaver      *screen_saver,
+                  GAsyncResult        *result,
+                  GsdMediaKeysManager *manager)
+{
+        gboolean is_locked;
+        GError *error = NULL;
+
+        is_locked = gsd_screen_saver_call_lock_finish (screen_saver, result, &error);
+
+        if (!is_locked) {
+                g_warning ("Couldn't lock screen: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+}
+
 static void
 do_lock_screensaver (GsdMediaKeysManager *manager)
 {
         GsdMediaKeysManagerPrivate *priv = manager->priv;
 
-        if (priv->connection == NULL) {
-                g_warning ("No existing D-Bus connection trying to handle screensaver lock key");
-                return;
-        }
-        g_dbus_connection_call (manager->priv->connection,
-                                GS_DBUS_NAME,
-                                GS_DBUS_PATH,
-                                GS_DBUS_INTERFACE,
-                                "Lock",
-                                NULL, NULL,
-                                G_DBUS_CALL_FLAGS_NONE, -1,
-                                NULL, NULL, NULL);
+        if (priv->screen_saver_proxy == NULL)
+                priv->screen_saver_proxy = gnome_settings_bus_get_screen_saver_proxy ();
+
+        gsd_screen_saver_call_lock (priv->screen_saver_proxy,
+                                    priv->cancellable,
+                                    (GAsyncReadyCallback) on_screen_locked,
+                                    manager);
 }
 
 static void
@@ -1511,9 +1533,11 @@ gsd_media_player_key_pressed (GsdMediaKeysManager *manager,
         have_listeners = (manager->priv->media_players != NULL);
 
         if (!have_listeners) {
-                /* Popup a dialog with an (/) icon */
-                show_osd (manager, "action-unavailable-symbolic", NULL, -1);
-                return TRUE;
+                if (!mpris_controller_key (manager->priv->mpris_controller, key)) {
+                        /* Popup a dialog with an (/) icon */
+                        show_osd (manager, "action-unavailable-symbolic", NULL, -1);
+                }
+		return TRUE;
         }
 
         player = manager->priv->media_players->data;
@@ -1898,7 +1922,7 @@ update_keyboard_cb (GObject             *source_object,
         }
 
         /* update the dialog with the new value */
-        g_variant_get (new_percentage, "(u)", &percentage);
+        g_variant_get (new_percentage, "(i)", &percentage);
         show_osd (manager, "keyboard-brightness-symbolic", NULL, percentage);
         g_variant_unref (new_percentage);
 }
@@ -1960,6 +1984,61 @@ do_battery_action (GsdMediaKeysManager *manager)
         g_free (label);
 }
 
+static void
+screencast_stop (GsdMediaKeysManager *manager)
+{
+        if (manager->priv->screencast_timeout_id > 0) {
+                g_source_remove (manager->priv->screencast_timeout_id);
+                manager->priv->screencast_timeout_id = 0;
+        }
+
+        g_dbus_proxy_call (manager->priv->screencast_proxy,
+                           "StopScreencast", NULL,
+                           G_DBUS_CALL_FLAGS_NONE, -1,
+                           manager->priv->screencast_cancellable,
+                           NULL, NULL);
+}
+
+static gboolean
+screencast_timeout (gpointer user_data)
+{
+        GsdMediaKeysManager *manager = user_data;
+        screencast_stop (manager);
+        return G_SOURCE_REMOVE;
+}
+
+static void
+screencast_start (GsdMediaKeysManager *manager)
+{
+        g_dbus_proxy_call (manager->priv->screencast_proxy,
+                           "Screencast",
+                           /* Translators: this is a filename used for screencast
+                            * recording, where "%d" and "%t" date and time, e.g.
+                            * "Screencast from 07-17-2013 10:00:46 PM.webm" */
+                           /* xgettext:no-c-format */
+                           g_variant_new_parsed ("(%s, @a{sv} {})",
+                                                 _("Screencast from %d %t.webm")),
+                           G_DBUS_CALL_FLAGS_NONE, -1,
+                           manager->priv->screencast_cancellable,
+                           NULL, NULL);
+
+        manager->priv->screencast_timeout_id = g_timeout_add_seconds (SCREENCAST_LENGTH,
+                                                                      screencast_timeout,
+                                                                      manager);
+}
+
+static void
+do_screencast_action (GsdMediaKeysManager *manager)
+{
+        if (manager->priv->screencast_proxy == NULL)
+                return;
+
+        if (manager->priv->screencast_timeout_id == 0)
+                screencast_start (manager);
+        else
+                screencast_stop (manager);
+}
+
 static void
 do_custom_action (GsdMediaKeysManager *manager,
                   guint                deviceid,
@@ -2035,6 +2114,9 @@ do_action (GsdMediaKeysManager *manager,
         case AREA_SCREENSHOT_CLIP_KEY:
                 gsd_screenshot_take (type);
                 break;
+        case SCREENCAST_KEY:
+                do_screencast_action (manager);
+                break;
         case WWW_KEY:
                 do_url_action (manager, "http", timestamp);
                 break;
@@ -2215,6 +2297,17 @@ on_shell_proxy_ready (GObject      *source,
                 g_dbus_proxy_new_for_bus_finish (result, NULL);
 }
 
+static void
+on_screencast_proxy_ready (GObject      *source,
+                           GAsyncResult *result,
+                           gpointer      data)
+{
+        GsdMediaKeysManager *manager = data;
+
+        manager->priv->screencast_proxy =
+                g_dbus_proxy_new_for_bus_finish (result, NULL);
+}
+
 static void
 on_key_grabber_ready (GObject      *source,
                       GAsyncResult *result,
@@ -2256,6 +2349,14 @@ on_shell_appeared (GDBusConnection   *connection,
                                   SHELL_DBUS_NAME,
                                   manager->priv->shell_cancellable,
                                   on_shell_proxy_ready, manager);
+
+        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                                  0, NULL,
+                                  name_owner,
+                                  SHELL_DBUS_PATH "/Screencast",
+                                  SHELL_DBUS_NAME ".Screencast",
+                                  manager->priv->screencast_cancellable,
+                                  on_screencast_proxy_ready, manager);
 }
 
 static void
@@ -2325,6 +2426,7 @@ start_media_keys_idle_cb (GsdMediaKeysManager *manager)
 
         ensure_cancellable (&manager->priv->grab_cancellable);
         ensure_cancellable (&manager->priv->shell_cancellable);
+        ensure_cancellable (&manager->priv->screencast_cancellable);
 
         manager->priv->name_owner_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
                                                          SHELL_DBUS_NAME, 0,
@@ -2332,6 +2434,9 @@ start_media_keys_idle_cb (GsdMediaKeysManager *manager)
                                                          on_shell_vanished,
                                                          manager, NULL);
 
+        g_debug ("Starting mpris controller");
+        manager->priv->mpris_controller = mpris_controller_new ();
+
         gnome_settings_profile_end (NULL);
 
         manager->priv->start_idle_id = 0;
@@ -2393,6 +2498,8 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
         g_clear_object (&priv->power_proxy);
         g_clear_object (&priv->power_screen_proxy);
         g_clear_object (&priv->power_keyboard_proxy);
+        g_clear_object (&priv->mpris_controller);
+        g_clear_object (&priv->screencast_proxy);
 
         if (manager->priv->name_owner_id) {
                 g_bus_unwatch_name (manager->priv->name_owner_id);
@@ -2428,6 +2535,11 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
                 g_clear_object (&priv->shell_cancellable);
         }
 
+        if (priv->screencast_cancellable != NULL) {
+                g_cancellable_cancel (priv->screencast_cancellable);
+                g_clear_object (&priv->screencast_cancellable);
+        }
+
         g_clear_object (&priv->sink);
         g_clear_object (&priv->source);
         g_clear_object (&priv->volume);
@@ -2562,6 +2674,8 @@ gsd_media_keys_manager_finalize (GObject *object)
         if (media_keys_manager->priv->inhibit_keys_fd != -1)
                 close (media_keys_manager->priv->inhibit_keys_fd);
 
+        g_clear_object (&media_keys_manager->priv->screen_saver_proxy);
+
         G_OBJECT_CLASS (gsd_media_keys_manager_parent_class)->finalize (object);
 }
 
