@@ -905,8 +905,6 @@ set_mousetweaks_daemon (GsdMouseManager *manager,
 
         if (! g_spawn_command_line_async (comm, &error)) {
                 if (error->code == G_SPAWN_ERROR_NOENT && run_daemon) {
-                        GtkWidget *dialog;
-
                         if (dwell_click_enabled) {
                                 g_settings_set_boolean (manager->priv->mouse_a11y_settings,
                                                         KEY_DWELL_CLICK_ENABLED, FALSE);
@@ -915,18 +913,7 @@ set_mousetweaks_daemon (GsdMouseManager *manager,
                                                         KEY_SECONDARY_CLICK_ENABLED, FALSE);
                         }
 
-                        dialog = gtk_message_dialog_new (NULL, 0,
-                                                         GTK_MESSAGE_WARNING,
-                                                         GTK_BUTTONS_OK,
-                                                         _("Could not enable mouse accessibility features"));
-                        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                                                  _("Mouse accessibility requires Mousetweaks "
-                                                                    "to be installed on your system."));
-                        gtk_window_set_title (GTK_WINDOW (dialog), _("Universal Access"));
-                        gtk_window_set_icon_name (GTK_WINDOW (dialog),
-                                                  "preferences-desktop-accessibility");
-                        gtk_dialog_run (GTK_DIALOG (dialog));
-                        gtk_widget_destroy (dialog);
+                        g_warning ("Error enabling mouse accessibility features (mousetweaks is not installed)");
                 }
                 g_error_free (error);
         }
@@ -1091,10 +1078,8 @@ touchpad_callback (GSettings       *settings,
                         continue;
 
                 if (g_str_equal (key, KEY_TAP_TO_CLICK)) {
-                        gboolean mouse_left_handed;
-                        mouse_left_handed = g_settings_get_boolean (manager->priv->mouse_settings, KEY_LEFT_HANDED);
                         set_tap_to_click (device, g_settings_get_boolean (settings, key),
-                                          get_touchpad_handedness (manager, mouse_left_handed));
+                                          g_settings_get_boolean (manager->priv->touchpad_settings, KEY_LEFT_HANDED));
                 } else if (g_str_equal (key, KEY_SCROLL_METHOD)) {
                         set_scroll_method (manager, device, g_settings_get_enum (settings, key));
                         set_horiz_scroll (device, g_settings_get_boolean (settings, KEY_PAD_HORIZ_SCROLL));
