@@ -348,7 +348,7 @@ static FixedEntry fixed_entries [] = {
         { "Gtk/ShowInputMethodMenu", fixed_false_int },
         { "Gtk/ShowUnicodeMenu",     fixed_false_int },
         { "Gtk/AutoMnemonics",       fixed_true_int },
-        { "Gtk/EnablePrimaryPaste",  fixed_false_int },
+        { "Gtk/EnablePrimaryPaste",  fixed_true_int },
 };
 
 static TranslationEntry translations [] = {
